@@ -5,11 +5,12 @@
 //!
 //! See: <https://docs.github.com/en/rest/security-advisories/global-advisories?apiVersion=2022-11-28>
 
-use anyhow::{Result, anyhow};
+use anyhow::anyhow;
 use github_actions_models::common::{RepositoryUses, Uses};
 
 use super::{Audit, AuditLoadError, audit_meta};
 use crate::{
+    audit::AuditError,
     config::Config,
     finding::{Confidence, Finding, Fix, Severity, location::Routable as _},
     github,
@@ -29,11 +30,11 @@ audit_meta!(
 );
 
 impl KnownVulnerableActions {
-    fn action_known_vulnerabilities(
+    async fn action_known_vulnerabilities(
         &self,
         uses: &RepositoryUses,
-    ) -> Result<Vec<(Severity, String, Option<String>)>> {
-        let version = match &uses.git_ref {
+    ) -> Result<Vec<(Severity, String, Option<String>)>, AuditError> {
+        let version = match &uses.git_ref() {
             // If `uses` is pinned to a symbolic ref, we need to perform
             // feats of heroism to figure out what's going on.
             // In the "happy" case the symbolic ref is an exact version tag,
@@ -51,9 +52,11 @@ impl KnownVulnerableActions {
             // To handle all of the above, we convert the ref into a commit
             // and then find the longest tag for that commit.
             version if !uses.ref_is_commit() => {
-                let Some(commit_ref) =
-                    self.client
-                        .commit_for_ref(&uses.owner, &uses.repo, version)?
+                let Some(commit_ref) = self
+                    .client
+                    .commit_for_ref(uses.owner(), uses.repo(), version)
+                    .await
+                    .map_err(Self::err)?
                 else {
                     // No `ref -> commit` means that the action's version
                     // is probably just outright invalid.
@@ -62,7 +65,9 @@ impl KnownVulnerableActions {
 
                 match self
                     .client
-                    .longest_tag_for_commit(&uses.owner, &uses.repo, &commit_ref)?
+                    .longest_tag_for_commit(uses.owner(), uses.repo(), &commit_ref)
+                    .await
+                    .map_err(Self::err)?
                 {
                     Some(tag) => tag.name,
                     // Somehow we've round-tripped through a commit and ended
@@ -79,7 +84,9 @@ impl KnownVulnerableActions {
             commit_ref => {
                 match self
                     .client
-                    .longest_tag_for_commit(&uses.owner, &uses.repo, commit_ref)?
+                    .longest_tag_for_commit(uses.owner(), uses.repo(), commit_ref)
+                    .await
+                    .map_err(Self::err)?
                 {
                     Some(tag) => tag.name,
                     // No corresponding tag means the user is maybe doing something
@@ -93,7 +100,9 @@ impl KnownVulnerableActions {
 
         let vulns = self
             .client
-            .gha_advisories(&uses.owner, &uses.repo, &version)?;
+            .gha_advisories(uses.owner(), uses.repo(), &version)
+            .await
+            .map_err(Self::err)?;
 
         let mut results = vec![];
 
@@ -120,14 +129,14 @@ impl KnownVulnerableActions {
     }
 
     /// Create a fix to upgrade to a specific non-vulnerable version
-    fn create_upgrade_fix<'doc>(
+    async fn create_upgrade_fix<'doc>(
         &self,
         uses: &RepositoryUses,
         target_version: String,
         step: &impl StepCommon<'doc>,
-    ) -> Result<Fix<'doc>> {
-        let mut uses_slug = format!("{}/{}", uses.owner, uses.repo);
-        if let Some(subpath) = &uses.subpath {
+    ) -> Result<Fix<'doc>, AuditError> {
+        let mut uses_slug = format!("{}/{}", uses.owner(), uses.repo());
+        if let Some(subpath) = &uses.subpath() {
             uses_slug.push_str(&format!("/{subpath}"));
         }
 
@@ -150,22 +159,27 @@ impl KnownVulnerableActions {
                 // convention or not, so we have to try both.
                 // We try the prefixed version first, since we expect it
                 // to be more common.
-                let (target_ref, target_commit) = self
+
+                let (target_ref, target_commit) = match self
                     .client
-                    .commit_for_ref(&uses.owner, &uses.repo, &prefixed_version)
-                    .map(|commit| commit.map(|commit| (&prefixed_version, commit)))
-                    .or_else(|_| {
-                        self.client
-                            .commit_for_ref(&uses.owner, &uses.repo, &bare_version)
-                            .map(|commit| commit.map(|commit| (&bare_version, commit)))
-                    })?
-                    .ok_or_else(|| {
-                        anyhow!(
-                            "Cannot resolve version {bare_version} to commit hash for {}/{}",
-                            uses.owner,
-                            uses.repo
-                        )
-                    })?;
+                    .commit_for_ref(uses.owner(), uses.repo(), &prefixed_version)
+                    .await
+                {
+                    Ok(commit) => commit.map(|commit| (&prefixed_version, commit)),
+                    Err(_) => self
+                        .client
+                        .commit_for_ref(uses.owner(), uses.repo(), &bare_version)
+                        .await
+                        .map_err(Self::err)?
+                        .map(|commit| (&bare_version, commit)),
+                }
+                .ok_or_else(|| {
+                    Self::err(anyhow!(
+                        "Cannot resolve version {bare_version} to commit hash for {}/{}",
+                        uses.owner(),
+                        uses.repo()
+                    ))
+                })?;
 
                 let new_uses_value = format!("{uses_slug}@{target_commit}");
 
@@ -194,7 +208,7 @@ impl KnownVulnerableActions {
                 // prefixed with `v` or not. Instead of trying to figure it out
                 // via the GitHub API, we match the style of the current `uses`
                 // clause.
-                let target_version_tag = if uses.git_ref.starts_with('v') {
+                let target_version_tag = if uses.git_ref().starts_with('v') {
                     prefixed_version
                 } else {
                     bare_version
@@ -214,14 +228,18 @@ impl KnownVulnerableActions {
         }
     }
 
-    fn process_step<'doc>(&self, step: &impl StepCommon<'doc>) -> Result<Vec<Finding<'doc>>> {
+    async fn process_step<'doc>(
+        &self,
+        step: &impl StepCommon<'doc>,
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         let Some(Uses::Repository(uses)) = step.uses() else {
             return Ok(findings);
         };
 
-        for (severity, id, first_patched_version) in self.action_known_vulnerabilities(uses)? {
+        for (severity, id, first_patched_version) in self.action_known_vulnerabilities(uses).await?
+        {
             let mut finding_builder = Self::finding()
                 .confidence(Confidence::High)
                 .severity(severity)
@@ -243,20 +261,21 @@ impl KnownVulnerableActions {
             // and only apply the highest one. This would be moderately annoying
             // to do, since we'd have to decide which finding to attach that
             // fix to.
-            if let Some(fix) = first_patched_version
-                .map(|patched_version| self.create_upgrade_fix(uses, patched_version, step))
-                .transpose()?
-            {
+            if let Some(first_patched_version) = first_patched_version {
+                let fix = self
+                    .create_upgrade_fix(uses, first_patched_version, step)
+                    .await?;
                 finding_builder = finding_builder.fix(fix);
             }
 
-            findings.push(finding_builder.build(step)?);
+            findings.push(finding_builder.build(step).map_err(Self::err)?);
         }
 
         Ok(findings)
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for KnownVulnerableActions {
     fn new(state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -275,16 +294,20 @@ impl Audit for KnownVulnerableActions {
             .map(|client| KnownVulnerableActions { client })
     }
 
-    fn audit_step<'doc>(&self, step: &Step<'doc>, _config: &Config) -> Result<Vec<Finding<'doc>>> {
-        self.process_step(step)
+    async fn audit_step<'doc>(
+        &self,
+        step: &Step<'doc>,
+        _config: &Config,
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
+        self.process_step(step).await
     }
 
-    fn audit_composite_step<'doc>(
+    async fn audit_composite_step<'doc>(
         &self,
         step: &CompositeStep<'doc>,
         _config: &Config,
-    ) -> Result<Vec<Finding<'doc>>> {
-        self.process_step(step)
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
+        self.process_step(step).await
     }
 }
 
@@ -314,8 +337,8 @@ mod tests {
         KnownVulnerableActions::new(&state).unwrap()
     }
 
-    #[test]
-    fn test_fix_upgrade_actions_checkout() {
+    #[tokio::test]
+    async fn test_fix_upgrade_actions_checkout() {
         let workflow_content = r#"
 name: Test Vulnerable Actions
 on: push
@@ -339,18 +362,17 @@ jobs:
         let step = &steps[0];
 
         // Test the fix directly
-        let uses = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "checkout".to_string(),
-            git_ref: "v2".to_string(),
-            subpath: None,
-        };
+        let uses = RepositoryUses::parse("actions/checkout@v2").unwrap();
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r#"
+
         name: Test Vulnerable Actions
         on: push
         jobs:
@@ -364,8 +386,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_actions_setup_node() {
+    #[tokio::test]
+    async fn test_fix_upgrade_actions_setup_node() {
         let workflow_content = r#"
 name: Test Node Setup
 on: push
@@ -391,18 +413,17 @@ jobs:
         let step = &steps[0];
 
         // Test the fix directly
-        let uses = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "setup-node".to_string(),
-            git_ref: "v1".to_string(),
-            subpath: None,
-        };
+        let uses = RepositoryUses::parse("actions/setup-node@v1").unwrap();
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
-        insta::assert_snapshot!(fixed_document.source(), @r#"
+        insta::assert_snapshot!(fixed_document.source(), @r"
+
         name: Test Node Setup
         on: push
         jobs:
@@ -415,11 +436,11 @@ jobs:
                   node-version: '18'
               - name: Install dependencies
                 run: npm install
-        "#);
+        ");
     }
 
-    #[test]
-    fn test_fix_upgrade_third_party_action() {
+    #[tokio::test]
+    async fn test_fix_upgrade_third_party_action() {
         let workflow_content = r#"
 name: Test Third Party Action
 on: push
@@ -445,18 +466,17 @@ jobs:
         let step = &steps[0];
 
         // Test the fix directly
-        let uses = RepositoryUses {
-            owner: "codecov".to_string(),
-            repo: "codecov-action".to_string(),
-            git_ref: "v1".to_string(),
-            subpath: None,
-        };
+        let uses = RepositoryUses::parse("codecov/codecov-action@v1").unwrap();
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r#"
+
         name: Test Third Party Action
         on: push
         jobs:
@@ -472,8 +492,8 @@ jobs:
         "#);
     }
 
-    #[test]
-    fn test_fix_upgrade_multiple_vulnerable_actions() {
+    #[tokio::test]
+    async fn test_fix_upgrade_multiple_vulnerable_actions() {
         let workflow_content = r#"
 name: Test Multiple Vulnerable Actions
 on: push
@@ -509,42 +529,31 @@ jobs:
         let audit = create_test_audit();
 
         // Fix checkout action
-        let uses_checkout = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "checkout".to_string(),
-            git_ref: "v2".to_string(),
-            subpath: None,
-        };
+        let uses_checkout = RepositoryUses::parse("actions/checkout@v2").unwrap();
         let fix_checkout = audit
             .create_upgrade_fix(&uses_checkout, "v4".into(), &steps[0])
+            .await
             .unwrap();
         current_document = fix_checkout.apply(&current_document).unwrap();
 
         // Fix setup-node action
-        let uses_node = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "setup-node".to_string(),
-            git_ref: "v1".to_string(),
-            subpath: None,
-        };
+        let uses_node = RepositoryUses::parse("actions/setup-node@v1").unwrap();
         let fix_node = audit
             .create_upgrade_fix(&uses_node, "v4".into(), &steps[1])
+            .await
             .unwrap();
         current_document = fix_node.apply(&current_document).unwrap();
 
         // Fix cache action
-        let uses_cache = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "cache".to_string(),
-            git_ref: "v2".to_string(),
-            subpath: None,
-        };
+        let uses_cache = RepositoryUses::parse("actions/cache@v2").unwrap();
         let fix_cache = audit
             .create_upgrade_fix(&uses_cache, "v4".into(), &steps[2])
+            .await
             .unwrap();
         current_document = fix_cache.apply(&current_document).unwrap();
 
-        insta::assert_snapshot!(current_document.source(), @r#"
+        insta::assert_snapshot!(current_document.source(), @r"
+
         name: Test Multiple Vulnerable Actions
         on: push
         jobs:
@@ -564,11 +573,11 @@ jobs:
                   key: ${{ runner.os }}-node-${{ hashFiles('**/package-lock.json') }}
               - name: Install dependencies
                 run: npm install
-        "#);
+        ");
     }
 
-    #[test]
-    fn test_fix_upgrade_action_with_subpath() {
+    #[tokio::test]
+    async fn test_fix_upgrade_action_with_subpath() {
         let workflow_content = r#"
 name: Test Action with Subpath
 on: push
@@ -592,18 +601,17 @@ jobs:
         let step = &steps[0];
 
         // Test the fix with subpath
-        let uses = RepositoryUses {
-            owner: "owner".to_string(),
-            repo: "repo".to_string(),
-            git_ref: "v1".to_string(),
-            subpath: Some("subpath".to_string()),
-        };
+        let uses = RepositoryUses::parse("owner/repo/subpath@v1").unwrap();
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v2".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v2".into(), step)
+            .await
+            .unwrap();
         let fixed_document = fix.apply(workflow.as_document()).unwrap();
 
         insta::assert_snapshot!(fixed_document.source(), @r"
+
         name: Test Action with Subpath
         on: push
         jobs:
@@ -617,8 +625,8 @@ jobs:
         ");
     }
 
-    #[test]
-    fn test_first_patched_version_priority() {
+    #[tokio::test]
+    async fn test_first_patched_version_priority() {
         // This test verifies that first_patched_version is used when available
         let workflow_content = r#"
 name: Test First Patched Version Priority
@@ -640,23 +648,20 @@ jobs:
         };
         let step = &steps[0];
 
-        let uses = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "checkout".to_string(),
-            git_ref: "v2".to_string(),
-            subpath: None,
-        };
+        let uses = RepositoryUses::parse("actions/checkout@v2").unwrap();
 
         // Test that when first_patched_version is provided, it's used
         let audit = create_test_audit();
         let fix_with_patched_version = audit
             .create_upgrade_fix(&uses, "v3.1.0".into(), step)
+            .await
             .unwrap();
         let fixed_document = fix_with_patched_version
             .apply(workflow.as_document())
             .unwrap();
 
-        insta::assert_snapshot!(fixed_document.source(), @r#"
+        insta::assert_snapshot!(fixed_document.source(), @r"
+
         name: Test First Patched Version Priority
         on: push
         jobs:
@@ -665,11 +670,11 @@ jobs:
             steps:
               - name: Vulnerable action
                 uses: actions/checkout@v3.1.0
-        "#);
+        ");
     }
 
-    #[test]
-    fn test_fix_symbolic_ref() {
+    #[tokio::test]
+    async fn test_fix_symbolic_ref() {
         let workflow_content = r#"
 name: Test Non-Commit Ref
 on: push
@@ -690,19 +695,18 @@ jobs:
         };
         let step = &steps[0];
 
-        let uses = RepositoryUses {
-            owner: "actions".to_string(),
-            repo: "checkout".to_string(),
-            git_ref: "v2".to_string(),
-            subpath: None,
-        };
+        let uses = RepositoryUses::parse("actions/checkout@v2").unwrap();
 
         let audit = create_test_audit();
-        let fix = audit.create_upgrade_fix(&uses, "v4".into(), step).unwrap();
+        let fix = audit
+            .create_upgrade_fix(&uses, "v4".into(), step)
+            .await
+            .unwrap();
 
         let new_doc = fix.apply(workflow.as_document()).unwrap();
 
         assert_snapshot!(new_doc.source(), @r"
+
         name: Test Non-Commit Ref
         on: push
         jobs:
@@ -714,8 +718,8 @@ jobs:
         ");
     }
 
-    #[test]
-    fn test_offline_audit_state_creation() {
+    #[tokio::test]
+    async fn test_offline_audit_state_creation() {
         // Test that we can create an audit state without a GitHub token
         let state = crate::state::AuditState::default();
 
@@ -725,8 +729,8 @@ jobs:
     }
 
     #[cfg(feature = "gh-token-tests")]
-    #[test]
-    fn test_fix_commit_pin() {
+    #[tokio::test]
+    async fn test_fix_commit_pin() {
         // Test with real GitHub API - requires GH_TOKEN environment variable
         let workflow_content = r#"
 name: Test Commit Hash Pinning Real API
@@ -760,11 +764,13 @@ jobs:
         let input = workflow.into();
         let findings = audit
             .audit(KnownVulnerableActions::ident(), &input, &Config::default())
+            .await
             .unwrap();
         assert_eq!(findings.len(), 1);
 
         let new_doc = findings[0].fixes[0].apply(input.as_document()).unwrap();
         assert_snapshot!(new_doc.source(), @r"
+
         name: Test Commit Hash Pinning Real API
         on: push
         permissions: {}
@@ -780,8 +786,8 @@ jobs:
     // TODO: test_fix_commit_pin_subpath
 
     #[cfg(feature = "gh-token-tests")]
-    #[test]
-    fn test_fix_commit_pin_no_comment() {
+    #[tokio::test]
+    async fn test_fix_commit_pin_no_comment() {
         // Ensure that we don't rewrite a version comment
         // if the `uses:` clause doesn't already have one.
         let workflow_content = r#"
@@ -815,11 +821,13 @@ jobs:
         let input = workflow.into();
         let findings = audit
             .audit(KnownVulnerableActions::ident(), &input, &Config::default())
+            .await
             .unwrap();
         assert_eq!(findings.len(), 1);
 
         let new_doc = findings[0].fixes[0].apply(input.as_document()).unwrap();
         assert_snapshot!(new_doc.source(), @r"
+
         name: Test Commit Hash Pinning Real API
         on: push
         permissions: {}
