@@ -4,13 +4,14 @@ use yamlpatch::{Op, Patch};
 
 use crate::{
     Confidence, Severity,
+    audit::AuditError,
     config::Config,
     finding::{
         Finding, Fix, FixDisposition, Persona,
         location::{Feature, Location, Routable},
     },
     models::{StepCommon, action::CompositeStep, workflow::Step},
-    utils::parse_fenced_expressions_from_input,
+    utils::{self, parse_fenced_expressions_from_input},
 };
 use subfeature::Subfeature;
 
@@ -32,7 +33,7 @@ impl Obfuscation {
         // GitHub happily interprets but otherwise gums up pattern matching
         // in audits like unpinned-uses, forbidden-uses, and cache-poisoning.
         // We check for some of these forms of nonsense here and report them.
-        if let Some(subpath) = uses.subpath.as_deref() {
+        if let Some(subpath) = uses.subpath() {
             for component in subpath.split('/') {
                 match component {
                     // . and .. are valid in uses subpaths, but are impossible to
@@ -59,7 +60,7 @@ impl Obfuscation {
 
     /// Normalizes a uses path by removing unnecessary components like empty slashes, `.`, and `..`.
     fn normalize_uses_path(&self, uses: &RepositoryUses) -> Option<String> {
-        let subpath = uses.subpath.as_deref()?;
+        let subpath = uses.subpath()?;
 
         let mut components = Vec::new();
         for component in subpath.split('/') {
@@ -82,14 +83,19 @@ impl Obfuscation {
 
         // If all components were removed, the subpath should be empty
         if components.is_empty() {
-            Some(format!("{}/{}@{}", uses.owner, uses.repo, uses.git_ref))
+            Some(format!(
+                "{}/{}@{}",
+                uses.owner(),
+                uses.repo(),
+                uses.git_ref()
+            ))
         } else {
             Some(format!(
                 "{}/{}/{}@{}",
-                uses.owner,
-                uses.repo,
+                uses.owner(),
+                uses.repo(),
                 components.join("/"),
-                uses.git_ref
+                uses.git_ref()
             ))
         }
     }
@@ -186,39 +192,70 @@ impl Obfuscation {
     fn process_step<'doc>(
         &self,
         step: &impl StepCommon<'doc>,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
-        if let Some(Uses::Repository(uses)) = step.uses() {
-            let obfuscated_annotations = self.obfuscated_repo_uses(uses);
-            if !obfuscated_annotations.is_empty() {
-                let mut finding_builder = Self::finding()
-                    .confidence(Confidence::High)
-                    .severity(Severity::Low);
+        match step.body() {
+            crate::models::StepBodyCommon::Uses {
+                uses: Uses::Repository(uses),
+                ..
+            } => {
+                let obfuscated_annotations = self.obfuscated_repo_uses(uses);
+                if !obfuscated_annotations.is_empty() {
+                    let mut finding_builder = Self::finding()
+                        .confidence(Confidence::High)
+                        .severity(Severity::Low);
 
-                // Add all annotations as locations
-                for annotation in &obfuscated_annotations {
-                    finding_builder = finding_builder.add_location(
-                        step.location()
-                            .primary()
-                            .with_keys(["uses".into()])
-                            .annotated(*annotation),
+                    // Add all annotations as locations
+                    for annotation in &obfuscated_annotations {
+                        finding_builder = finding_builder.add_location(
+                            step.location()
+                                .primary()
+                                .with_keys(["uses".into()])
+                                .annotated(*annotation),
+                        );
+                    }
+
+                    // Try to create a fix for the obfuscated uses path
+                    if let Some(fix) = self.create_uses_fix(uses, step) {
+                        finding_builder = finding_builder.fix(fix);
+                    }
+
+                    findings.push(finding_builder.build(step).map_err(Self::err)?);
+                }
+            }
+            crate::models::StepBodyCommon::Run { .. } => {
+                if let Some(("cmd" | "cmd.exe", shell_loc)) = step
+                    .shell()
+                    .map(|(shell, loc)| (utils::normalize_shell(shell), loc))
+                {
+                    // `shell: cmd` is basically impossible to analyze: it has no formal
+                    // grammar and has several line continuation mechanisms that stymie
+                    // naive matching. It also hasn't been the default shell on Windows
+                    // runners since 2019.
+                    findings.push(
+                        Self::finding()
+                            .confidence(Confidence::High)
+                            .severity(Severity::Low)
+                            .add_location(
+                                step.location_with_grip()
+                                    .annotated("Windows CMD shell limits analysis"),
+                            )
+                            .add_location(shell_loc.primary())
+                            .tip("use 'shell: pwsh' or 'shell: bash' for improved analysis")
+                            .build(step)
+                            .map_err(Self::err)?,
                     );
                 }
-
-                // Try to create a fix for the obfuscated uses path
-                if let Some(fix) = self.create_uses_fix(uses, step) {
-                    finding_builder = finding_builder.fix(fix);
-                }
-
-                findings.push(finding_builder.build(step)?);
             }
+            _ => {}
         }
 
         Ok(findings)
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for Obfuscation {
     fn new(_state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -227,11 +264,11 @@ impl Audit for Obfuscation {
         Ok(Self)
     }
 
-    fn audit_raw<'doc>(
+    async fn audit_raw<'doc>(
         &self,
         input: &'doc AuditInput,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         for (expr, expr_span) in parse_fenced_expressions_from_input(input) {
@@ -289,26 +326,26 @@ impl Audit for Obfuscation {
                     }
                 }
 
-                findings.push(finding_builder.build(input)?);
+                findings.push(finding_builder.build(input).map_err(Self::err)?);
             }
         }
 
         Ok(findings)
     }
 
-    fn audit_step<'doc>(
+    async fn audit_step<'doc>(
         &self,
         step: &Step<'doc>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         self.process_step(step)
     }
 
-    fn audit_composite_step<'a>(
+    async fn audit_composite_step<'a>(
         &self,
         step: &CompositeStep<'a>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'a>>> {
+    ) -> Result<Vec<Finding<'a>>, AuditError> {
         self.process_step(step)
     }
 }
@@ -322,7 +359,7 @@ mod tests {
     use crate::state::AuditState;
 
     /// Helper function to apply a fix and return the result for snapshot testing
-    fn apply_fix_for_snapshot(workflow_content: &str, _audit_name: &str) -> String {
+    async fn apply_fix_for_snapshot(workflow_content: &str, _audit_name: &str) -> String {
         let key = InputKey::local("dummy".into(), "test.yml", None::<&str>);
         let workflow = Workflow::from_string(workflow_content.to_string(), key).unwrap();
         let audit_state = AuditState {
@@ -332,6 +369,7 @@ mod tests {
         let audit = Obfuscation::new(&audit_state).unwrap();
         let findings = audit
             .audit_workflow(&workflow, &Default::default())
+            .await
             .unwrap();
 
         assert!(!findings.is_empty(), "Expected findings but got none");
@@ -355,8 +393,8 @@ mod tests {
         fixed_document.source().to_string()
     }
 
-    #[test]
-    fn test_obfuscation_fix_uses_path_empty_components() {
+    #[tokio::test]
+    async fn test_obfuscation_fix_uses_path_empty_components() {
         let workflow_content = r#"
 name: Test Workflow
 on: push
@@ -368,8 +406,9 @@ jobs:
       - uses: actions/checkout////@v4
 "#;
 
-        let result = apply_fix_for_snapshot(workflow_content, "obfuscation");
-        insta::assert_snapshot!(result, @r#"
+        let result = apply_fix_for_snapshot(workflow_content, "obfuscation").await;
+        insta::assert_snapshot!(result, @r"
+
         name: Test Workflow
         on: push
 
@@ -378,11 +417,11 @@ jobs:
             runs-on: ubuntu-latest
             steps:
               - uses: actions/checkout@v4
-        "#);
+        ");
     }
 
-    #[test]
-    fn test_obfuscation_fix_uses_path_dot() {
+    #[tokio::test]
+    async fn test_obfuscation_fix_uses_path_dot() {
         let workflow_content = r#"
 name: Test Workflow
 on: push
@@ -394,8 +433,9 @@ jobs:
       - uses: github/codeql-action/./init@v2
 "#;
 
-        let result = apply_fix_for_snapshot(workflow_content, "obfuscation");
-        insta::assert_snapshot!(result, @r#"
+        let result = apply_fix_for_snapshot(workflow_content, "obfuscation").await;
+        insta::assert_snapshot!(result, @r"
+
         name: Test Workflow
         on: push
 
@@ -404,11 +444,11 @@ jobs:
             runs-on: ubuntu-latest
             steps:
               - uses: github/codeql-action/init@v2
-        "#);
+        ");
     }
 
-    #[test]
-    fn test_obfuscation_fix_uses_path_double_dot() {
+    #[tokio::test]
+    async fn test_obfuscation_fix_uses_path_double_dot() {
         let workflow_content = r#"
 name: Test Workflow
 on: push
@@ -420,8 +460,9 @@ jobs:
       - uses: actions/cache/save/../save@v4
 "#;
 
-        let result = apply_fix_for_snapshot(workflow_content, "obfuscation");
-        insta::assert_snapshot!(result, @r#"
+        let result = apply_fix_for_snapshot(workflow_content, "obfuscation").await;
+        insta::assert_snapshot!(result, @r"
+
         name: Test Workflow
         on: push
 
@@ -430,6 +471,6 @@ jobs:
             runs-on: ubuntu-latest
             steps:
               - uses: actions/cache/save@v4
-        "#);
+        ");
     }
 }
