@@ -185,7 +185,7 @@ impl TemplateInjection {
             return None;
         }
 
-        let shell = utils::normalize_shell(step.shell()?);
+        let shell = utils::normalize_shell(step.shell()?.0);
 
         match shell {
             "bash" | "sh" | "zsh" => Some(format!("${{{env_var}}}")),
@@ -707,6 +707,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -758,6 +759,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -811,6 +813,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -920,6 +923,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Multiple Template Injections
                 on: push
                 jobs:
@@ -986,6 +990,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -1046,6 +1051,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -1109,6 +1115,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -1225,6 +1232,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Bash
                     on: push
                     jobs:
@@ -1271,6 +1279,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Bash
                     on: push
                     jobs:
@@ -1316,7 +1325,8 @@ jobs:
                         finding,
                         "replace expression with environment variable",
                     );
-                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    insta::assert_snapshot!(fixed_content.source(), @r"
+
                     name: Test Template Injection - CMD
                     on: push
                     jobs:
@@ -1326,7 +1336,7 @@ jobs:
                           - name: Vulnerable step with cmd shell
                             shell: cmd
                             run: echo User is %GITHUB_ACTOR%
-                    "#);
+                    ");
                 }
             }
         );
@@ -1363,6 +1373,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - PowerShell
                     on: push
                     jobs:
@@ -1409,6 +1420,7 @@ jobs:
                     );
                     // Ubuntu default shell is bash, so should use ${VAR} syntax
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Default Shell Ubuntu
                     on: push
                     jobs:
@@ -1454,6 +1466,7 @@ jobs:
                     );
                     // Windows default shell is pwsh, so should use $env:VAR syntax
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Default Shell Windows
                     on: push
                     jobs:
@@ -1498,7 +1511,8 @@ jobs:
                         finding,
                         "replace expression with environment variable",
                     );
-                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    insta::assert_snapshot!(fixed_content.source(), @r"
+
                     name: Test Template Injection - CMD with Custom Env
                     on: push
                     jobs:
@@ -1510,7 +1524,7 @@ jobs:
                             run: echo PR title is %GITHUB_EVENT_PULL_REQUEST_TITLE%
                             env:
                               GITHUB_EVENT_PULL_REQUEST_TITLE: ${{ github.event.pull_request.title }}
-                    "#);
+                    ");
                 }
             }
         );
